use std::borrow::BorrowMut;

use isahc::{ReadResponseExt, Request, RequestExt};
use serde::Deserialize;

use crate::BASE_URL;

#[derive(Deserialize)]
struct CountsCounts {
    posts: u64,
}

#[derive(Deserialize)]
struct CountsReponse {
    counts: CountsCounts,
}

pub fn count(tags: String) -> Result<u64, String> {
    let web_safe_tags: String = url::form_urlencoded::byte_serialize(tags.as_bytes()).collect();

    let mut count_response = Request::get::<String>(format!(
        "{}/counts/posts.json?tags={}",
        BASE_URL, web_safe_tags
    ))
    .body(())
    .expect("Couldn't get body")
    .send();

    let text_try = match count_response.borrow_mut() {
        Ok(res) => res.text(),
        Err(err) => return Err(err.to_string()),
    };

    let text = match text_try {
        Ok(text) => text,
        Err(_) => return Err("Couldn't parse count body".into()),
    };

    let parsed = match serde_json::from_str::<CountsReponse>(&text) {
        Ok(parsed) => parsed,
        Err(e) => return Err(format!("Couldn't parse count text: {}", e.to_string()).into()),
    };

    Ok(parsed.counts.posts)
}
